/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "fps.h"

static struct timeval time_begin;
static struct timeval time_end;

#ifdef LIMIT_FPS
static struct itimerval my_itimer;
static volatile int pending=0;
#endif

static volatile unsigned long loops;

#ifndef LIMIT_FPS

void init_fps () { gettimeofday ( &time_begin , NULL ); loops = 0; }
void next_fps () { loops++; }

#else

void ack_mytimer(int signum) 
{ pending = 0; 
  (void)signal( SIGALRM, ack_mytimer );
  // catch spurious SIGALRM, ignore Errors!!
};
// signals always return to default behaviour when raised!
// re-set with signal() if you need to catch that signal again

void start_mytimer()
{ if (pending)
  { // komisch, duerfte nicht passieren...
    fprintf(stderr,"Signal pending when activating iTIMER ... (PID %i)\n",\
            getpid());
    pending = 0; // das muss reichen
  };
  my_itimer.it_interval.tv_sec = 0;  // zum Neustarten des Timers
  my_itimer.it_interval.tv_usec = 0; // 0 bedeutet "nicht neu starten"
  my_itimer.it_value.tv_sec = 0;     // erste Wartezeit
  my_itimer.it_value.tv_usec = (int)(1000000.0 / LIMIT_FPS);
  pending = 0;
  if (setitimer( ITIMER_REAL , &my_itimer , NULL ))
  { perror(strerror(errno));
    fprintf(stderr," - Konnte Timer nicht zur fps-Begrenzung verwenden\n");
    pending = 0;
  }
  else 
  { // sigaction ist POSIX, signal ist ANSI, SIG{KILL,STOP} sind nie handlebar
    // SIG{FPE,ILL,SEGV} sind kritisch - danach besser Programm beenden.
    // sigaction ist deutlich komplizierter bzw vielseitiger
       // -> ermoeglicht Polling mit sigpending und Warten mit sigsuspend
    if (signal(SIGALRM, ack_mytimer) == SIG_ERR) 
    { err_exit("Error installing SIGALRM handler!?\n");
    };
    pending = 1; // Erst setzen, wenn ack garantiert arbeitet!
  };
  // ITIMER_REAL: real time -> SIGALRM
  // ITIMER_VIRTUAL: only when process executes -> SIGVTALRM
  // ITIMER_PROF: only when p. e. or sys does sth for p. -> SIGPROF
};


void init_fps () 
{ gettimeofday ( &time_begin , NULL ); 
  loops = 0;
  if (pending) 
  { pending = 0; 
    fprintf(stderr, "Was ist los? Der FPS-Count beginnt doch erst...\n");
  }; // Das muss reichen - was immer der Grund war.
  pending = 0;
  start_mytimer ();
};

void next_fps () 
{ double rest;
  int skala;

  loops++;
  if (pending) 
  { getitimer( ITIMER_REAL , &my_itimer );
    skala = my_itimer.it_value.tv_usec;
    rest = 0.0;
    if (skala)
    { (void)pause ();
      // pause always returns -1, and errno is set  to  ERESTARTNOHAND.
      // Errors: EINTR  signal was received.
      if (skala<1001) skala = 0; // nicht ueber winzige Rest-CPU-Zeiten jubeln
      rest = (double)skala / (1000000.0 / (LIMIT_FPS * 100.0));
#ifdef CPU_WRITE
      fprintf(stderr, "%f%% CPU uebrig\n", rest);
#endif
    };// Rechenzeit ist uebrig :-)
#ifdef CPU_SKALA
    rest = rest * 0.01 * (W-2.0);
    skala = 0;
    for (; skala<(round2int(rest)); skala++)
    { GFX( skala , 0 , buffer ) = RGB ( 0 , 255 , 0 ); 
    };
    for (; skala<W; skala++) 
    { GFX( skala , 0, buffer ) = RGB ( 255 , 0 , 0 );
    };
#endif
  };
  start_mytimer (); // PS: raise(SIG) kann diesen Prozess ein Sig schicken
};

#endif

void show_fps (char * subname) {

  unsigned long duration;
  double fps;

#ifdef LIMIT_FPS
  if (pending) 
  { (void)signal( SIGALRM , ack_mytimer );
//    fprintf(stderr,"\nWarte CPU-Restzeit des letzten Frames ab...\n");
    fprintf(stderr,"...");
    pause (); 
  };// hoffentlich wurde init_fps schon gerufen...
  
  pending = 0; // wenn jetzt noch nicht, weiss ich auch nicht.
  (void)signal( SIGALRM , ack_mytimer );
  // SIG_IGN ignores, SIG_DFL resets to default (SIGALRM: terminate process)
  // (returns previous handler)
#endif
  
  gettimeofday ( &time_end , NULL );
  duration =
    ( (time_end.tv_sec  - time_begin.tv_sec ) * 1000000 ) +
      (time_end.tv_usec - time_begin.tv_usec);
  duration = (duration>0) ? duration : 1;
  fps = 1000000.0;
  fps = fps * (double)loops / duration;

#ifndef LIMIT_FPS  
  printf ( "%s->%s [%dx%d]: %g fps\n" , PROG , subname , W , H , fps );
#else
  printf ( "%s->%s [%dx%d]: %g of %g fps\n" , PROG , subname , W , H , fps , \
           (double)LIMIT_FPS );
#endif
#ifdef USE_OSS
  printf ( "Zur Zeit haben wir OSS Tick Nummer %i (Frequenz %g)\n", \
           gettime(), (double)HZ);
#endif

}
