/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 24bit
 * colors and 2-3 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "grafik.h"
#ifndef SCALE
#define SCALE 1
#endif

#ifdef GAG
#define GAG_ 3
#else
#define GAG_ 1
#endif

/* X-Window-Code geklaut und angepasst von "guimauve2" */

void init_x () {

  dis = XOpenDisplay  ( NULL );
  disDepth = -1;
  if ( dis != NULL)
    {
      disScreen = DefaultScreen ( dis );
      disDepth  = DefaultDepth  ( dis , disScreen );
      disWidth  = DisplayWidth  ( dis , disScreen );
      disHeight = DisplayHeight ( dis , disScreen );
    }
    
  if ( disDepth != 24 )
    {
      fprintf ( stderr , "Fehler: Farbtiefe ist %d, nicht 24\n" , disDepth );
      if ( disDepth > 0 ) XCloseDisplay  ( dis );
      exit (1);
    }

  winRoot                  = DefaultRootWindow ( dis );
  winAttr.border_pixel     = BlackPixel ( dis , disScreen );
  winAttr.background_pixel = BlackPixel ( dis , disScreen );
  winMask                  = CWBackPixel | CWBorderPixel;
 
  win = XCreateWindow ( dis , winRoot , X , Y , W * SCALE * GAG_ , H * SCALE ,
                        BORDER , disDepth ,
			InputOutput , CopyFromParent , winMask , &winAttr );

  XStoreName ( dis , win , NAME );

  XSelectInput ( dis , win , KeyPressMask | KeyReleaseMask
    | ButtonPressMask | ButtonReleaseMask );

  winHint.flags = PPosition | PMinSize | PMaxSize ;
  winHint.x = X;                     
  winHint.y = Y;
  winHint.max_width  = winHint.min_width  = W * SCALE * GAG_;
  winHint.max_height = winHint.min_height = H * SCALE;
  XSetWMNormalHints ( dis , win , &winHint );

  XClearWindow ( dis , win );
  XMapRaised ( dis , win );
  XFlush ( dis );

  buffer = ( EA_PIXEL * ) malloc ( sizeof(EA_PIXEL) * W * H );
  buffer_byte = ( unsigned char * ) buffer;
  buffer2 = (SCALE == 1) ? 
    buffer : 
    ( EA_PIXEL * ) malloc ( sizeof(EA_PIXEL) * W * SCALE * H * SCALE );
  if ((buffer2 == NULL) || (buffer == NULL))
    { 
      fprintf ( stderr , "Fehler: No RAM for X-Buffers!\n" );
      XDestroyWindow ( dis , win );
      XCloseDisplay  ( dis );
      exit(1);
    }
  xim = XCreateImage ( dis , CopyFromParent , disDepth , ZPixmap , 0 ,
		       ( void * ) buffer2 , W * SCALE , H * SCALE , 
		       8 * sizeof(EA_PIXEL) , W * sizeof(EA_PIXEL) * SCALE );
		       /* last Args: bits per pixel in RAM and XSize(bytes) */

  gcVal.foreground = 0;     
  gcVal.background = 0;
  gcMask           = GCForeground | GCBackground;
  gc = XCreateGC ( dis , win , gcMask , &gcVal );

}


int event_x () {

  XEvent XEv;

  return ( ! XCheckWindowEvent ( dis , win ,
    KeyReleaseMask || ButtonReleaseMask , &XEv ) );
  /* if event found, it is moved to &XEv, and true is returned */
  /* so event_x() returns true UNTIL such an event is found.   */
}


void close_x () {

  XFreeGC        ( dis , gc );
  XDestroyImage  ( xim );
  XDestroyWindow ( dis , win );
  XCloseDisplay  ( dis );     
  free( buffer ); /* buffer_byte is just an alias to this */
  free( buffer2 );
}

void show_x () {

  if (SCALE == 1)
  { XPutImage ( dis , win , gc , xim , 0 , 0 , 0     , 0     , W , H );
// gag shows same image 3 times next to each other
#ifdef GAG
    XPutImage ( dis , win , gc , xim , 0 , 0 , W     , 0     , W , H );
    XPutImage ( dis , win , gc , xim , 0 , 0 , W + W , 0 * H , W , H );
#endif
  } // Ende SCALE == 1
  else
  { if ( (sizeof(EA_PIXEL) != 2) || (SCALE != 2) ) 
       // nur fuer Scale 2 und 15/16bit ist bisher Schnell-Code fertig
    { int x, y, x2, y2, n;
      EA_PIXEL now;
      for (y=0, y2=0;  y < (W * H);  y+= W , y2+= (W * SCALE) )
      { for (x=0, x2=0; x < W; x++)
        { now = buffer[y + x];
          for (n=0; n < SCALE; n++) { buffer2[y2 + x2] = now; x2++; };
        };
        for (n=1; n < SCALE; n++)
        { memcpy((void *)(buffer2 + y2 + (W * SCALE)) , \
                 (void *)(buffer2 + y2) , W * SCALE * sizeof(EA_PIXEL));
          y2 += W * SCALE;
        };
      };
    } 
    else // Glueck gehabt - Schnell-Code kann verwendet werden:
    { __asm__("pushl %%ebp"
         "cld"
              "movl %%ecx,%%ebx"
         ".align 16"
      "2: movl %%edi,%%ebp"
      "1: lodsw; stosw; stosw"
         "loop 1b"
           "movl %%esi,%%eax"
         "movl %%ebp,%%esi"
              "movl %%ebx,%%ecx; "
         "rep; movsl"
           "movl %%eax,%%esi"
              "movl %%ebx,%%ecx"
         "decl %%edx"
         "jnz 2b"
         "popl %%ebp"
              "::D(buffer2),S(buffer),c(W),d(H)"
              ":esi,edi,eax,ecx,edx,ebx");
      // Funktionsweise: alle Pixel verdoppelt kopieren,
      // dann ganze Ziel-Zeile verdoppeln (PS: 1b = 1, backward)
      // und das Ganze fuer jede Zeile wiederholen
      // ebx ist Backup von ecx,  eax ist Pixel bzw. Backup von esi
      // ebp ist Backup eines Zielzeilenanfangs
      // ebp wird von Hand gesichert - GCC 2.7.2 macht das nicht korrekt!?!
    }; // Ende Schnell-Code
    
      XPutImage ( dis , win , gc , xim , 0 , 0 , 0             , 0 , W * SCALE , H * SCALE );
// gag shows same image 3 times next to each other
#ifdef GAG
      XPutImage ( dis , win , gc , xim , 0 , 0 , W * SCALE     , 0 , W * SCALE , H * SCALE ); 
      XPutImage ( dis , win , gc , xim , 0 , 0 , W * SCALE * 2 , 0 , W * SCALE , H * SCALE ); 
#endif
    }; // Ende SCALE != 1
};
