/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "laser.h"

/* Lasershow_Polar_Lissajous AuerSoft 5/97, Idee MM, 8.6.97 for use as .H */
/* Version fuer X11 3/2000 */
// typen.h hat round2int(a) als ((int)rint(a)) definiert

void laser_step(int first, int xsize, int ysize, EA_PIXEL * dest)
{
  const int la_update_peri=96;
  const int la_stepsize=96;
#define RAD_PERI (6 * 48)
  double mod_f = PI2 * 0.00001;
  double max_phi_inc = M_PI_2 / RAD_PERI; // M_PI_2 defined as PI/2 in math.h
 static double phi=0;
 static int nput=0;
 static int rphi=0;
 static long la_cnt; 
 static int xbuf[LA_TRAIL]; 
 static int ybuf[LA_TRAIL];
 static EA_PIXEL * ziel;
 static EA_PIXEL laserpal[LA_COLORS];
 static EA_PIXEL bgcolor;
 static double rad_sintab_x[RAD_PERI];
 static double rad_sintab_y[RAD_PERI];
 static int xhalf;
 static int yhalf;
 double phi_inc;
 int x, y, cnt; 

if (first == 1)
{ nput = 0;
  rphi = 0;
  for (cnt=0; cnt<LA_TRAIL; cnt++) { xbuf[cnt] = 0; ybuf[cnt] = 0; };
  la_cnt = 0;
  ziel = dest;
  bgcolor = RGB(32, 32, 32);
  phi_inc =  PI2 / LA_COLORS;  // PI2 is 2*PI, defined in math.h
  for (cnt=0; cnt<LA_COLORS; cnt++) 
  { laserpal[cnt] = RGB( \
      128 + round2int(sin(phi - (PI * 0.667)) * 127.0), \
      128 + round2int(sin(phi) * 127.0), \
      128 + round2int(sin(phi + (PI * 0.667)) * 127.0) );
    phi += phi_inc;       // ^- PI is defined in math.h
  };
  for (y=0; y<ysize; y++)
  { for (x=0; x<xsize; x++) { GFX(x, y, ziel) = bgcolor; }
  }; /* setting everything to bgcolor! */
  xhalf = xsize >> 1;
  yhalf = ysize >> 1;
  for (y=0, phi=0; y<RAD_PERI; y++, phi += PI2 / RAD_PERI)
  { phi_inc = sin(phi) * 0.95;
    rad_sintab_x[y] = phi_inc * xhalf;
    rad_sintab_y[y] = phi_inc * yhalf;
  };
  phi = 0.0; 
};

if (first == 0) 
{ phi_inc = max_phi_inc * PI * sin(mod_f * la_cnt);
  if ((mod_f * la_cnt) >= PI2) la_cnt -= (int)( PI2 / mod_f ); // sicherer...
  for (cnt=0; cnt<la_stepsize; cnt++)
  { if ((la_cnt % la_update_peri) == 0)
      phi_inc = max_phi_inc * PI * sin(mod_f * la_cnt);
    la_cnt++;
    round2int_inline(x, sin(phi) * rad_sintab_x[rphi]);
    x += xhalf;
    round2int_inline(y, cos(phi) * rad_sintab_y[rphi]);
    y += yhalf;
    GFX(xbuf[nput], ybuf[nput], ziel) = bgcolor;
    xbuf[nput] = x;
    ybuf[nput] = y;
    GFX(x, y, ziel) = laserpal[la_cnt % LA_COLORS];
    nput++;
    if (nput >= LA_TRAIL) nput=0;
    rphi++;
    if (rphi >= RAD_PERI) rphi=0;
    phi += phi_inc;
    if (phi >= PI2) phi-=PI2;
  }; /* for */
}

if (first == -1) 
{ /* hide trail */
  for (cnt=0; cnt<LA_TRAIL; cnt++) 
  { GFX(xbuf[cnt], ybuf[cnt], ziel) = bgcolor; };
};

/* Anwendung: 
   1. laser_step(1,xsize,ysize,&grafikpuffer[index_ecke_linksoben]);
   2. while (...) { laser_step(0,egal,egal,egal); ... anzeigen(); };
   3. laser_step(-1,0,0,NULL);
*/

return;

};
