/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "musik.h"
#include "stdlib.h" /* ausnahmsweise echtes rand() */

#define MYpow pow

#define Beat    (SampRate>>2)
// also 1/4 Sec = "480bpm"
#define BeatPause (Beat>>2)
#define BeatsPerMus 8
#define MusLen  (BeatsPerMus*Beat)

#define InstLen 420
#define Oktaven 3
#define Noten   (12*Oktaven)

#ifdef SOUND16BIT
/* if ByPerSamp is 2 */
#define STILL 0x8000
#define AMPLI 0x3300
#define SAMP unsigned short int
#else
/* if ByPerSamp is 1 */
#define STILL 0x80
#define AMPLI 0x33
#define SAMP unsigned char
#endif

static int takt = -1;             // HIER...

static SAMP musik[MusLen*StereoChn]; // Ausgabe
static SAMP instrument1[InstLen]; // Sample
static SAMP instrument2[InstLen]; // Sample
static double noten[Noten];       // Multiplikationstabelle
static int melodie1[32] =         // ... UND HIER die Musik beeinflussen?
  { 24,12,19,15, 24, 0,19, 0, 24,12,19,15, 24, 0,15,19,
    15,19,15,19, 22, 0,15,19, 14,17,21,14, 22,17,17,22 };
  // erstmal geklaut vom 3x666 Intro...
static int melodie2[32] =         // ... UND HIER die Musik beeinflussen?
  { 12, 0,24,12, 12,24,12,24, 12, 0,24,12, 12,24,12,24,
    15, 0,27,15, 15,27,15,27, 14, 0,26,14, 15,27,17,29 };
  // erstmal geklaut vom 3x666 Intro...
  
static int nested=0;

void musikfueller()
{  
  int x,y, x2,y2;
  SAMP * sampp;
  double freq;
  double wobinich_f;
  int wobinich_i;
  double root12of2;

  if (nested) 
     { for (x=0,y=0; x<MusLen*StereoChn; x++) y*=sqrt(musik[x]);
       fprintf(stderr,"Nested call to musikfueller() !?\n");
       return;
     };
  nested=1;
  
  if (takt < 0)
  { for (x=0; x<MusLen*StereoChn; x++) { musik[x] = STILL; };
    root12of2 = MYpow( 2.0 , 1.0/12.0 );
    freq = (double)InstLen * 22.5  / (double)SampRate; 
      // Ausgabesamples pro Periode ->
      // Inkrement im Instrumentensample pro Ausgabesample
    for ( x=0;  x<Noten;  x++, freq*=root12of2 ) { noten[x] = freq;  };
    // 2 Instrumente, je 50% Amplitude
    wobinich_f = (double)(STILL - (AMPLI * 0.7));
    freq = (double)(AMPLI+AMPLI) * 0.7 / (double)InstLen;
    for ( x=0;  x<InstLen;  x++, wobinich_f+=freq) 
        { instrument1[x] = round2int(wobinich_f) >> 1; 
        }; // einfach nur Saegezahn (wirkt lauter, also 50% leiser machen)
    freq = 2.0 * PI2 / InstLen;
    wobinich_f = 0.0;
    for ( x=0;  x<InstLen;  x++, wobinich_f+=freq) 
        { instrument2[x] = 
          round2int((double)STILL + ((double)AMPLI * sin(wobinich_f))) >> 1; 
        }; // einfach nur Sinus, 1 Oktave hoeher
    takt = 0;
    fprintf(stderr,"Musik startet...\n");
    nested = 0;
    return; // genug fuer den Anfang
  };
  
  sampp = &musik[0];
  y = (takt & 3) << 3;
  // Takt waehlt eine von 4 8er-Gruppen aus der Melodietabelle
  for ( y2=y+8;  y<y2;  y++ ) 
  { freq = noten[melodie1[y]];
    wobinich_f = 0.0;
    for ( x=0;  x<(Beat-BeatPause);  x++, wobinich_f+=freq, sampp++ ) //
        { round2int_inline( wobinich_i , wobinich_f );                //
          if (wobinich_i >= InstLen)                                  //
             { wobinich_i = 0; wobinich_f -= InstLen; };              //
          *sampp = instrument1[wobinich_i];                           //
          if (StereoChn==2) {
              sampp++;
              *sampp = instrument1[wobinich_i];                       //
          }
        };
      // mit Hardware koennte man hier gut interpolieren etc. ...
      // Subsampling sozusagen, um mehr aus kuerzeren Samples zu machen!
    for ( ; x<Beat;  x++, sampp++ )  //
        { *sampp = STILL;            //
          if (StereoChn==2) {
              sampp++;
              *sampp = STILL;
          }
        };
  };

  sampp = &musik[0];
  y = (takt & 3) << 3;
  // Takt waehlt eine von 4 8er-Gruppen aus der 2. Melodietabelle
  for ( y2=y+8;  y<y2;  y++ ) 
  { freq = noten[melodie2[y]];
    wobinich_f = 0.0;
    for ( x=0;  x<(Beat-BeatPause);  x++, wobinich_f+=freq, sampp++ ) //
        { round2int_inline( wobinich_i , wobinich_f );                //
          if (wobinich_i >= InstLen)                                  //
             { wobinich_i = 0; wobinich_f -= InstLen; };              //
          *sampp += instrument2[wobinich_i];                          //
          if (StereoChn==2) {
              sampp++;
              *sampp = instrument1[wobinich_i];                       //
          }
        };
    for ( ; x<Beat;  x++, sampp++ )  //
        {  /*   *sampp = STILL;   */ //
          /* if (StereoChn==2) { */
          /*    sampp++; */
          /*    *sampp = STILL; */
          /* } */
        };
  };
  
  oss_send(musik,sizeof(musik));
  takt++;
  
  // Mutation:
  if (!(takt % 3))
  { for (y2=0; y2<7; y2++)
    { x = rand() & 31; 
      y = rand() & 31;
      if (rnd(0,1) == 1)
      { x2 = melodie1[x]; melodie1[x]=melodie1[y]; melodie1[y] = x2; }
      else
      { x2 = melodie2[x]; melodie2[x]=melodie1[y]; melodie1[y] = x2; }
    };
  };

#if 0
  fprintf(stderr,"%i ",takt);
  /* "Fortschritts-Anzeige": Taktnummer auf stderr ausgeben */
#endif

  nested = 0;
};
