/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "starfield_3d.h"

/* Starfield using simple 3D, AuerSoft 22.6.97, X11 Version 3/2000     */
/* DOS Version war 8.8 Festkomma und zbuf und colbuf waren 1 Feld =8-| */
/* Starmaker bestimmt die x/y Koords bei maximaler Annaeherung,        */
/* die deutlich ausserhalb des Fensters liegen koennen (!).            */
/* Spaeter wird x/y dann mit 1/z gezoomt und z geht als loop downto 1  */
// round2int(a) siehe typen.h -> ((int)rint(a))
// jetzt auch als Assembler: round2int_inline(intvalue,floatvalue) 8-)

void star_step(int first, int xsize, int ysize, EA_PIXEL * dest)
{
  const int maxdepth = 100;
  const int mediumdist = 66;
  const int smalldist = 33;
 static int xbuf[STARS]; 
 static int ybuf[STARS]; 
 static int   zbuf[STARS];
 static EA_PIXEL colbuf[STARS];
 static EA_PIXEL * st_dest;
 static EA_PIXEL starpal[ST_COLORS];
 static int xhalf, yhalf, xmax, ymax;
 int cnt, cc;
 int x, y, z; 
 float zinv; 
 EA_PIXEL c; 
 static EA_PIXEL bgcolor;
 static EA_PIXEL color_msb;

#define STARMAKER(id) { zbuf[id] = maxdepth; \
 colbuf[id] = starpal[rnd(0, ST_COLORS-1)]; \
 xbuf[id] = rnd(-(xhalf<<7), xhalf<<7); \
 ybuf[id] = rnd(-(yhalf<<7), yhalf<<7); \
 if ( (abs(xbuf[id]) < 1) || (abs(ybuf[id]) < 1) ) zbuf[id]=0; }

#define BIGPIX(col) { GFX(x, y, st_dest)=col; GFX(x+1, y, st_dest)=col; \
 GFX(x, y+1, st_dest)=col; GFX(x+1, y+1, st_dest)=col; }

if (first == 1) 
{ xhalf = xsize >> 1;
  xmax = xsize - 2; /* wg BIGPIX */
  yhalf = ysize >> 1;
  ymax = ysize - 2; /* wg BIGPIX */
  for (cnt=0; cnt<STARS; cnt++) 
  { STARMAKER(cnt); 
    /* Zuerst sollen nicht alle Sterne gleich fern sein, also: */
    zbuf[cnt] = rnd(1, maxdepth);
  };
  for (cnt=0; cnt < (ST_COLORS-3); cnt += 4) 
  { cc = rnd(64,127); // max 50% Helligkeit wg. Trick ...
    starpal[cnt] = RGB(cc, cc, cc);          // Graustufe
    starpal[cnt+1] = RGB(cc, cc, 0);         // Gelbton
    starpal[cnt+2] = RGB(cc>>1, cc>>1, 127); // Strahleblau
    starpal[cnt+2] = RGB(cc, cc>>1, cc>>1);  // Blassrot
  };
  st_dest=dest; 
  bgcolor = RGB(0, 0, 0);
  color_msb = RGB(128, 128, 128);
  for (y=0; y<ysize; y++)
  { for (x=0; x<xsize; x++) { GFX(x, y, st_dest) = bgcolor; };
  }; /* alles auf bgcolor setzen! */
};

if (first == 0) 
{ x = 0; y = 0;
  for (cnt=0; cnt<STARS; cnt++)
  { z = zbuf[cnt]; 
    if (z > 0)
    { zinv = 1.0 / z;
      round2int_inline( x , zinv * xbuf[cnt] ); x += xhalf;
      round2int_inline( y , zinv * ybuf[cnt] ); y += yhalf;
//    x = xhalf + round2int( zinv * xbuf[cnt] );
//    y = yhalf + round2int( zinv * ybuf[cnt] );
      if ((x < 0) || (y < 0) || \
          (x >= xmax) || (y >= ymax) )
      { /* clipping - out of range */ }
      else
      { if (z >= smalldist) 
        { GFX(x, y, st_dest) = bgcolor; 
        } else { BIGPIX(bgcolor); };
      };
      z--; 
      zbuf[cnt] = z; 
    };
    if (z > 0) { zinv = 1.0 / z;
                 round2int_inline( x , zinv * xbuf[cnt] ); x += xhalf; 
                 round2int_inline( y , zinv * ybuf[cnt] ); y += yhalf; 
//               x = xhalf + round2int( zinv * xbuf[cnt] );
//               y = yhalf + round2int( zinv * ybuf[cnt] );
               };
    if ((z < 1) || (x < 0) || (y < 0) || \
        (x >= xmax) || (y >= ymax) ) 
    { STARMAKER(cnt); /* diesmal ist z-Wert ok... */ }
    else 
    { c = colbuf[cnt];
      if ( (z < mediumdist) && (!(c & color_msb)) )
      { c <<= 1; colbuf[cnt] = c; };
      // bei mittlerer Entfernung Helligkeit maximal hochshiften!
      if (z >= smalldist) 
      { GFX(x, y, st_dest) = c; // Kleines Pixel...
      } else { BIGPIX(c); };    // ... oder grosses Pixel!
    };
  };
};

if (first == -1) 
{ for (cnt=0; cnt<STARS; cnt++) // hide all stars
  { z = zbuf[cnt]; 
    if (z > 0)
    { zinv = 1.0 / z;
      round2int_inline( x , zinv * xbuf[cnt] ); x += xhalf; 
      round2int_inline( y , zinv * ybuf[cnt] ); y += yhalf;
//    x = xhalf + round2int( zinv * xbuf[cnt] );
//    y = yhalf + round2int( zinv * ybuf[cnt] );
      if ((x < 0) || (y < 0) || \
          (x >= xmax) || (y >= ymax) )
      { /* clipping - out of range */ }
      else
      { if (z >= smalldist) 
        { GFX(x, y, st_dest)=bgcolor; 
        } else { BIGPIX(bgcolor); };
      };
    };
  };
};

};
