/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

#include "testmuster.h"

/* Version 3/2000 beschleunigt aber unuebersichtlicher ... */
/* sogar mit lookuptable statt sqrt(0.5 * ((xf*xf)+yf) );  */
/* dsp2derive nun getrennt, modulator also von bel. Quelle */

// abs (int->int!) / fabs (double->double) / labs (long int->long int)

void testmuster (int first, int wwx, int hhx, EA_PIXEL * zielx, float modulator)
{ int x, y, yrel, negx, negy, ymulwwdiv2;
  float xf, yf, xrel2, yrel2;
  float modul_x_step;
  // float = (gfloat is defined as float ) = 
  // double = (gdouble and float64 are defined as double) = 8by 
  // extended = long double = (gldouble is defined as ...) = 10by
  // uebrigens: [u_]int[8|16|32|64]_t kommen von char, short, int, long
  static float inv2hh_ar;
  static float inv2ww;
  static int wwdiv2;
  static int hh_ardiv2;
  static int ww;
  static int hh;
  static int hh_ar;
  static EA_PIXEL * ziel;
  static EA_PIXEL * rotius = NULL;
  static EA_PIXEL gruenius[(W>>1)+4];
  int R, G, B; 
  EA_PIXEL thispixel;

  if (first == 1)
  { ww = wwx;
    hh = hhx;
    ziel = zielx;
    hh_ar = minimum(ww,hh); // besser mit aspect ratio...
    wwdiv2 = ww >> 1;
    hh_ardiv2 = hh_ar >> 1;
    rotius = (EA_PIXEL *)malloc (wwdiv2 * hh_ardiv2 * sizeof(EA_PIXEL) + 4);
    /* wenn rotius == NULL, Pech - ohne Tabelle gehts auch, aber lahmer */
    if (rotius != NULL)
    { inv2hh_ar = 2.0 / hh_ar;
      inv2ww = 2.0 / ww;
      for (y=0; y<hh_ardiv2; y++)
      { ymulwwdiv2 = y * wwdiv2;
        yf = inv2hh_ar * (hh_ardiv2 - y);
        yf *= yf;
        xf = 1.0;
        for (x=0; x<wwdiv2; x++)
        { round2int_inline( R, 255.0 * ( 1.0 - sqrt( 0.5 * ((xf*xf)+yf) ) ) );
          rotius[ymulwwdiv2 + x] = RGB( CLIP2BYTE(R) , 0 , 0 );
          xf -= inv2ww;
        };
      }
    }; /* Ende rotius Berechnung */
    inv2hh_ar = 2.0 / hh_ar;
    inv2ww = 2.0 / ww;
  };

  if (first == -1)
  {
    free(rotius); 
    /* evtl. noch Clearscreen? Lieber nicht! */
    return;
  };
  
  if (modulator < 0.1) { modulator = 0.1; };
  if (modulator > 10.0) { modulator = 10.0; };
  modul_x_step = 255.0 * modulator / wwdiv2;

  if ((rotius != NULL) && (ww <= W)) // dann beschleunigte Anzeige :-)
  { 
    xrel2 = 255.0 * modulator;
    for (x=0; x < wwdiv2; x++) // Precalc fuer alle Zeilen gleich
    { round2int_inline( G , xrel2 );
      gruenius[x]  = RGB( 0 , CLIP2BYTE( G ) , 0 );
      // round2int_inline und CLIP2BYTE ist schneller als CLIPFLOAT2BYTE
      xrel2 -= modul_x_step; 
    };
    for (y=0, negy=hh_ar-1, yrel=hh_ardiv2; y < hh_ardiv2; y++, negy--, yrel--)
    { yf = inv2hh_ar * yrel;
      yf *= yf; // square-opcode ist nicht ueber math.h erreichbar
      ymulwwdiv2 = y * wwdiv2;
      yrel2 = (255.0 / (float)hh_ardiv2) * (float)yrel; 
      yrel2 /= modulator;
    
      round2int_inline( B, yrel2 );
      B = CLIP2BYTE(B);
      
      for (x=0, negx=ww-1; x < wwdiv2; x++, negx--)
      { GFX( x    , y    , ziel) = 
        GFX( negx , y    , ziel) =
        GFX( x    , negy , ziel) =
        GFX( negx , negy , ziel) = 
          rotius[ymulwwdiv2 + x] | gruenius[x] | RGB( 0 , 0 , B );
      };
    };
  }    // Ende Version mit Beschleunigung

  else

  { 
    for (y=0; y < hh_ardiv2; y++) 
    { negy = (hh_ar - 1) - y;
      yrel = hh_ardiv2 - y;
      yf = inv2hh_ar * yrel;
      ymulwwdiv2 = y * wwdiv2;
      yrel2 = (255.0 / hh_ardiv2) * yrel; // dabei auch int -> float
      yrel2 /= modulator;
      yf *= yf; /* math.h bietet kein sqr */
      // math.h bietet uebrigens hypot(x,y) = sqrt(x*x + y*y) ...
    
      round2int_inline( B, yrel2 );
      B = CLIP2BYTE(B);
      negx = ww - 1;
      xf = 1.0;
      xrel2 = 255.0 * modulator;
      for (x=0; x < wwdiv2; x++) // ohne Tabelle langsame Anzeige :-(
      { __asm__(" movl %eax,%eax; ");
        round2int_inline( R, 255.0 * ( 1.0 - sqrt((xf*xf) + yf) ) );
                                       /* this is zf.. */
        /* hier kann R auf 1.0-sqrt(2.0) schrumpfen, sieht anders aus! */
        __asm__(" movl %eax,%eax; ");
        round2int_inline( G, xrel2 ); 
        thispixel = RGB( CLIP2BYTE(R) , CLIP2BYTE(G) , B );
        GFX( x    , y    , ziel) = thispixel; 
        GFX( negx , y    , ziel) = thispixel;
        GFX( x    , negy , ziel) = thispixel;
        GFX( negx , negy , ziel) = thispixel;
        xrel2 -= modul_x_step;
        xf -= inv2ww;
        negx--;
      };
    };
  }; // Ende Version ohne Beschleunigung

};

/* ----------------------------------------------------------------- */

#ifdef DSPTEST

float dsp2derive(int fileno) // 0 ist z.B. stdin
{
#ifndef DEBUG
# define DEBUG 0
#endif
  int vol, inputlen, x;
  static int peak;
  float peak_converted,ableitung;
    static float peak_c_old=0.0;
    static float peak_c_older=0.0;
    static float peak_c_veryold=0.0;
  static unsigned char inputdata[DSPTEST];
  
  if ((fcntl(fileno , F_SETFL , O_NONBLOCK)==-1) && DEBUG)
    perror(sys_errlist[errno]);
  /* damit read () -1 und errno=EAGAIN gibt, wenn gerade nix zu lesen ist... */
  /* geht mit cat ... | FIRE aber komischerweise nicht mit FIRE </dev/dsp !? */

  if (((inputlen = read(fileno , inputdata , DSPTEST))==-1) && DEBUG)
    perror(sys_errlist[errno]);
  if (DEBUG) printf("%i of %i\n",inputlen,DSPTEST);

  if (inputlen>0)
  {
  peak = 0;
  for (x=0 ; (x < minimum(inputlen,1000)) ; x++)
    { /* gedacht fuer /dev/dsp, unsigned bytes Audio */
      vol = inputdata[x];
      vol = vol - 128 ; 
      vol = abs(vol) << 1; // mehr als 1 bedeutet gain... fuer lin also 2 oder 3
      peak = (vol>peak) ? vol : peak;
    }
  } else
  {
    // peak -= (peak >> 4);
  }
  peak_converted = log((float)peak + M_E ) * ( 1.0 / log(7.0 * 255.0));
  // faktor auf 255.0 ist bei log der anti-gain, z.B 7.0
  // lin: (float)peak * ( 1.0 / 255.0 )
    ableitung = 0.042 + (peak_converted - 
                (0.33 * (peak_c_old + peak_c_older + peak_c_veryold)));
    ableitung *= 8.4; 
    // ok, getrickst weil gesmoothet. dann gleich noch mehr Tricks ok...
    // also Shift und Scale nach Geschmack.
    peak_c_veryold = peak_c_older;
    peak_c_older = peak_c_old;
    peak_c_old = peak_converted;
  return ableitung;
}

#endif
