/*
 * This is FXcube, a tiny Linux demo where I collected my favourite
 * oldskool effects on a single 3d spinning cube! Have fun. To skip
 * scenes, press keys. Some 100 MHz needed. You need X11 with 16bit
 * colors and 1.5 MB of free RAM, plus /dev/dsp for fake music :-).
 * Copyright 1994-2003 by Eric Auer <eric@coli.uni-sb.de>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should find a copy of the GNU General Public License on the
 * site where you have downloaded this source. If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307 USA
 * (or try http://www.gnu.org/licenses/licenses.html at www.gnu.org).
 */

/* to use warper, make effects write to warpinbuf, not buffer */
/* warpinbuf -> do_warper() -> buffer -> show_x() -> buffer2 -> (X11) */

#include "warper.h"
#include "textur.h"

void init_warper(void) {
  warpinbuf = ( EA_PIXEL * ) malloc ( sizeof(EA_PIXEL) * W * H );
} /* init_warper */

void close_warper(void) {
  free( warpinbuf );
} /* close_warper */

/*
 * Copy warpinbuf to buffer, assuming that warpinbuf is split
 * into 4 columns and 2 rows of textures. The fields are used
 * as follows:
 * LEFT RIGHT TOP   BOTTOM
 * ONE  TWO   THREE FOUR
 * L/R are zoomed by factor 2 to form the background.
 * T/B and 1..4 are used to draw a 3d CUBE which is
 * rotated as specified by phi1 and phi2.
 */
void do_warper(int phi1, int phi2) {
  EA_PIXEL *target;
  EA_PIXEL *source;
  EA_PIXEL pix;
  int x, y;
  
  /* copy LEFT tile and zoom by factor 2 */
  for (y=0; y<(H>>1); y++) {
    source = &GFX(0, y, warpinbuf);
    target = &GFX(0, y+y, buffer);
    for (x=0; x<(W>>2); x++) {
      pix = *(source++);
      *(target++) = pix;
      *(target++) = pix;
    }
    source = &GFX(0, y+y, buffer);
    target = &GFX(0, y+y+1, buffer);
    for (x=0; x<(W>>1); x++)
      *(target++) = *(source++);
  }

  /* copy RIGHT tile and zoom by factor 2 */
  for (y=0; y<(H>>1); y++) {
    source = &GFX(W>>2, y, warpinbuf);
    target = &GFX(W>>1, y+y, buffer);
    for (x=0; x<(W>>2); x++) {
      pix = *(source++);
      *(target++) = pix;
      *(target++) = pix;
    }
    source = &GFX(W>>1, y+y, buffer);
    target = &GFX(W>>1, y+y+1, buffer);
    for (x=0; x<(W>>1); x++)
      *(target++) = *(source++);
  }

  /* copy ALL into a thumbnail of 1/8 size */
  for (y=0; y<(H>>3); y++) {
    source = &GFX(0, y<<3, warpinbuf);
    target = &GFX(7*(W>>3), y + (7*(H>>3)), buffer);
    for (x=0; x<(W>>3); x++) {
      *(target++) = *source;
      source += 8;
    }
  }

  /* the main cube drawing happens here: */
  cube(phi1, phi2, warpinbuf, buffer);
  
} /* do_warper */
